@@ -1,3 +1,9 @@
+# -- Server --
+
+#: The server name (scheme + domain + port)
+SERVER_NAME = ''
+
+
 # -- Action mapping --
 
 #: The mapping from request methods to resource actions, which is
@@ -10,17 +16,23 @@ ACTION_MAP = {
     'DELETE': 'delete',
     'OPTIONS': 'options',
     'HEAD': 'head',
-    'TRACE': 'trace',
+    'TRACE': 'trace'
 }
 
 
 # -- Parser and Renderer --
 
-#: The default Parser class.
-PARSER_CLASS = 'restart.parser.JSONParser'
+#: The default Parser classes.
+PARSER_CLASSES = (
+    'restart.parsers.JSONParser',
+    'restart.parsers.URLEncodedParser',
+    'restart.parsers.MultiPartParser',
+)
 
-#: The default Renderer class.
-RENDERER_CLASS = 'restart.renderer.JSONRenderer'
+#: The default Renderer classes.
+RENDERER_CLASSES = (
+    'restart.renderers.JSONRenderer',
+)
 
 
 # -- Logger --
