@@ -3,8 +3,10 @@ from __future__ import absolute_import
 from werkzeug.utils import import_string
 
 from .config import config
-from .exceptions import HTTPException
+from .negotiator import Negotiator
 from .response import Response
+from . import exceptions
+from .utils import locked_cached_classproperty
 
 
 class Resource(object):
@@ -16,21 +18,45 @@ class Resource(object):
     #: The name of the resource.
     name = None
 
-    #: The class used for parser objects.
-    parser_class = import_string(config.PARSER_CLASS)
-
-    #: The class used for parser objects.
-    renderer_class = import_string(config.RENDERER_CLASS)
-
-    #: The instances of middleware classes.
-    middlewares = tuple(
-        import_string(middleware_class)()
-        for middleware_class in config.MIDDLEWARE_CLASSES
+    #: The parser classes used for parsing request data.
+    parser_classes = tuple(
+        import_string(parser_class)
+        for parser_class in config.PARSER_CLASSES
     )
 
+    #: The renderer classes used for rendering response data.
+    renderer_classes = tuple(
+        import_string(renderer_class)
+        for renderer_class in config.RENDERER_CLASSES
+    )
+
+    #: The class used to select the proper parser or renderer
+    negotiator_class = Negotiator
+
+    #: The resource-level middleware classes
+    middleware_classes = ()
+
     def __init__(self, action_map):
         self.action_map = action_map
 
+    @locked_cached_classproperty(name='_middlewares')
+    def middlewares(cls):
+        """The instances of all middleware classes.
+
+        The final order of middleware classes:
+            The first is global middleware classes (in order) and
+            then is resource-level middleware classes (in order).
+        """
+        global_middleware_classes = tuple(
+            import_string(middleware_class_string)
+            for middleware_class_string in config.MIDDLEWARE_CLASSES
+        )
+        middleware_classes = global_middleware_classes + cls.middleware_classes
+        return tuple(
+            middleware_class()
+            for middleware_class in middleware_classes
+        )
+
     @property
     def logger(self):
         """A :class:`logging.Logger` object for this API."""
@@ -68,7 +94,10 @@ class Resource(object):
         :param args: the positional arguments captured from the URI.
         :param kwargs: the keyword arguments captured from the URI.
         """
-        self.request = request.parse(self.parser_class)
+        negotiator = self.negotiator_class()
+        self.format_suffix = kwargs.pop('format', None)
+
+        self.request = request.parse(negotiator, self.parser_classes)
         self.log_message('<Request> %s' % request.data)
 
         try:
@@ -79,7 +108,16 @@ class Resource(object):
         response = self.make_response(rv)
         self.log_message('<Response> %s %s' % (response.status, response.data))
 
-        return response.render(self.renderer_class)
+        return response.render(negotiator, self.renderer_classes,
+                               self.format_suffix)
+
+    def http_method_not_allowed(self, request, *args, **kwargs):
+        """The default action handler if the corresponding action for
+        `request.method` is not implemented.
+
+        See :meth:`dispatch_request` for the meanings of the parameters.
+        """
+        raise exceptions.MethodNotAllowed()
 
     def find_action(self, request):
         """Find the appropriate action according to the request method.
@@ -94,12 +132,7 @@ class Resource(object):
             )
             raise
 
-        try:
-            action = getattr(self, action_name)
-        except AttributeError as exc:
-            exc.args = ('Unimplemented action %r' % action_name,)
-            raise
-
+        action = getattr(self, action_name, self.http_method_not_allowed)
         return action
 
     def perform_action(self, *args, **kwargs):
@@ -171,13 +204,13 @@ class Resource(object):
 
         :param exc: the exception to be handled.
         """
-        if isinstance(exc, HTTPException):
+        if isinstance(exc, exceptions.HTTPException):
             headers = dict(exc.get_headers(self.request.environ))
             rv = ({'message': exc.description}, exc.code, headers)
             return rv
         else:
             self.log_exception(exc)
-            raise exc
+            raise
 
     def make_response(self, rv):
         """Converts the return value to a real response object that is
